//*****************************************************************************
//
//	UART1 des ATmega Interrupt gesteuert
//
//*****************************************************************************
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdlib.h>
#include <avr/wdt.h>
#include <string.h>
#include "UART1.h"
#include "watchdog.h"


#define BUF1_LAENGE  128

char c_RXBUF1[BUF1_LAENGE+1];
char c_TXBUF1[BUF1_LAENGE+1];
  
unsigned int ui_RX1cnt,ui_TX1cnt,ui_RX1wr,ui_TX1wr,ui_RX1rd,ui_TX1rd;


//*****************************************************************************
//	RX - INT
//*****************************************************************************
ISR( USART1_RX_vect)
{
   c_RXBUF1[ui_RX1wr]  = UDR1;
   ui_RX1wr++;
  
   if (ui_RX1wr > BUF1_LAENGE) ui_RX1wr = 0;
   ui_RX1cnt++; 
}

//*****************************************************************************
//	TX - INT  USART_UDRE_vect
//*****************************************************************************
ISR (USART1_UDRE_vect )
{
  if (ui_TX1cnt)
   {
      UDR1 = c_TXBUF1[ui_TX1rd];
	  ui_TX1rd++;
	  if (ui_TX1rd > BUF1_LAENGE) ui_TX1rd = 0;
	  if (ui_TX1cnt > 0) ui_TX1cnt--; 
   }
//   else UCSR1B &= 0xDF;
}


//*****************************************************************************
//	UART - Initialisierung
//	Input:	-
//	Output:	-
//*****************************************************************************
void UART1_INIT(void)
{
unsigned char x;
unsigned int i;

   SREG &= 0x7F;				//global INT aus
	                   
   ui_RX1cnt = 0;
   ui_TX1cnt = 0;
   ui_RX1wr  = 0;
   ui_TX1wr  = 0;
   ui_RX1rd  = 0;
   ui_TX1rd  = 0;
	
   for(i=0;i<BUF1_LAENGE+1;i++)
	{
	   c_TXBUF1[i] = 0;
	   c_RXBUF1[i] = 0;
       watchdog_reset();
	}
	
	// DDRE 	= 0xFE;  
	// PORTE 	= 0x00; 	

   UCSR1A = 0;    //Int Flag loeschen
   UCSR1B = 0;
   UCSR1C = 0x0E;
   UBRR1  = 103;				// 9k6 Baudrate (103 bei 16MHz)
   //UBRR1  =  25;				//38k4 Baudrate ( 25 bei 16MHz)

   UCSR1B = 0x98;				//RX & TX der UART an, Interrupts enabled	
	
   x = UDR1;

   while((UCSR1A & 0x20) == 0) //UDRE == 0 ?
    {
       watchdog_reset();
    }

   while((UCSR1A & 0x40) == 0x40) //TXC  == 1 ?
    {
       watchdog_reset();
    }

   UDR1 = 0x0D;					//sende CR, damit TXC flag setzen
   SREG |= 0x80;				//global INT an
}
 
//*****************************************************************************
//	UART - Test, ob Zeichen empfangen
//	Input:	-
//	Output:	1 ja , 0 nein
//*****************************************************************************
unsigned char UART1_RXTest(void)
{
   if (ui_RX1cnt) return(1); 
   	 else return(0);
}


//*****************************************************************************
//	UART - Zeichen senden
//	Input:	char  Zeichen
//	Output:	-
//*****************************************************************************
void UART1_TX(char c_x)
{
   while(ui_TX1cnt > 0) //warte bis Puffer leer
   {
      watchdog_reset();
   }

   SREG &= 0x7F;  //INTs sperren  

   c_TXBUF1[ui_TX1wr] = c_x;
   ui_TX1wr++;
   if (ui_TX1wr > BUF1_LAENGE) ui_TX1wr = 0;
   ui_TX1cnt++; 
   if (ui_TX1cnt > BUF1_LAENGE) ui_TX1cnt = 0; 

   SREG |= 0x80;  //INTs freigeben  

   UCSR1B |= 0x20; 	     
}


//*****************************************************************************
//	UART - String senden
//	Input:	char* String 
//	Output:	-
//*****************************************************************************
void UART1_TXStr(char * pc_str)
{
   while(ui_TX1cnt > 0) //warte bis Puffer leer
   {
      watchdog_reset();
   }

   while (*pc_str != '\0')
   {
      c_TXBUF1[ui_TX1wr] = *pc_str;
      ui_TX1wr++;
      if (ui_TX1wr > BUF1_LAENGE) ui_TX1wr = 0;
      SREG &= 0x7F;  //INTs sperren  
      ui_TX1cnt++; 
      if (ui_TX1cnt > BUF1_LAENGE) ui_TX1cnt = 0;
      SREG |= 0x80;    //INTs freigeben  
  	  pc_str++; 
      watchdog_reset();
   }
   
  UCSR1B |= 0x20; 	 
}


//*****************************************************************************
//	UART - Zeichen empfangen
//	Input:	- 
//	Output:	char  Empfangszeichen
//*****************************************************************************
char UART1_RX(void)
{      
char uc_x;
         
   while(ui_RX1cnt == 0)		//warte auf Zeichen
   {
      watchdog_reset();
   }

   uc_x = c_RXBUF1[ui_RX1rd];
  
   SREG &= 0x7F;  //INTs sperren  
   ui_RX1rd++;
   if (ui_RX1rd > BUF1_LAENGE) ui_RX1rd = 0;
   if (ui_RX1cnt > 0) ui_RX1cnt--;  
   SREG |= 0x80;  //INTs freigeben  
  
   return(uc_x);
}


