//*****************************************************************************
//
//	UART2 des ATmega Interrupt gesteuert
//
//*****************************************************************************
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdlib.h>
#include <avr/wdt.h>
#include <string.h>
#include "UART2.h"
#include "watchdog.h"


#define BUF2_LAENGE  128

char c_RXBUF2[BUF2_LAENGE+1];
char c_TXBUF2[BUF2_LAENGE+1];
  
unsigned int ui_RX2cnt,ui_TX2cnt,ui_RX2wr,ui_TX2wr,ui_RX2rd,ui_TX2rd;


//*****************************************************************************
//	RX - INT
//*****************************************************************************
ISR( USART2_RX_vect)
{
   c_RXBUF2[ui_RX2wr]  = UDR2;
   ui_RX2wr++;
  
   if (ui_RX2wr > BUF2_LAENGE) ui_RX2wr = 0;
   ui_RX2cnt++; 
}

//*****************************************************************************
//	TX - INT  USART_UDRE_vect
//*****************************************************************************
ISR (USART2_UDRE_vect )
{
  if (ui_TX2cnt)
   {
      UDR2 = c_TXBUF2[ui_TX2rd];
	  ui_TX2rd++;
	  if (ui_TX2rd > BUF2_LAENGE) ui_TX2rd = 0;
	  if (ui_TX2cnt > 0) ui_TX2cnt--; 
   }
//   else UCSR2B &= 0xDF;
}


//*****************************************************************************
//	UART - Initialisierung
//	Input:	-
//	Output:	-
//*****************************************************************************
void UART2_INIT(void)
{
unsigned char x;
unsigned int i;

   SREG &= 0x7F;				//global INT aus
	                   
   ui_RX2cnt = 0;
   ui_TX2cnt = 0;
   ui_RX2wr  = 0;
   ui_TX2wr  = 0;
   ui_RX2rd  = 0;
   ui_TX2rd  = 0;
	
   for(i=0;i<BUF2_LAENGE+1;i++)
	{
	   c_TXBUF2[i] = 0;
	   c_RXBUF2[i] = 0;
       watchdog_reset();
	}
	
	// DDRE 	= 0xFE;  
	// PORTE 	= 0x00; 	

   UCSR2A = 0;    //Int Flag loeschen
   UCSR2B = 0;
   UCSR2C = 0x0E;
   UBRR2  = 103;				// 9k6 Baudrate (103 bei 16MHz)
   //UBRR2  =  25;				//38k4 Baudrate ( 25 bei 16MHz)

   UCSR2B = 0x98;				//RX & TX der UART an, Interrupts enabled	
	
   x = UDR2;

   while((UCSR2A & 0x20) == 0) //UDRE == 0 ?
    {
       watchdog_reset();
    }

   while((UCSR2A & 0x40) == 0x40) //TXC  == 1 ?
    {
       watchdog_reset();
    }

   UDR2 = 0x0D;					//sende CR, damit TXC flag setzen
   SREG |= 0x80;				//global INT an
}
 
//*****************************************************************************
//	UART - Test, ob Zeichen empfangen
//	Input:	-
//	Output:	1 ja , 0 nein
//*****************************************************************************
unsigned char UART2_RXTest(void)
{
   if (ui_RX2cnt) return(1); 
   	 else return(0);
}


//*****************************************************************************
//	UART - Zeichen senden
//	Input:	char  Zeichen
//	Output:	-
//*****************************************************************************
void UART2_TX(char c_x)
{
   while(ui_TX2cnt > 0) //warte bis Puffer leer
   {
      watchdog_reset();
   }

   SREG &= 0x7F;  //INTs sperren  

   c_TXBUF2[ui_TX2wr] = c_x;
   ui_TX2wr++;
   if (ui_TX2wr > BUF2_LAENGE) ui_TX2wr = 0;
   ui_TX2cnt++; 
   if (ui_TX2cnt > BUF2_LAENGE) ui_TX2cnt = 0; 

   SREG |= 0x80;  //INTs freigeben  

   UCSR2B |= 0x20; 	     
}


//*****************************************************************************
//	UART - String senden
//	Input:	char* String 
//	Output:	-
//*****************************************************************************
void UART2_TXStr(char * pc_str)
{
   while(ui_TX2cnt > 0) //warte bis Puffer leer
   {
      watchdog_reset();
   }

   while (*pc_str != '\0')
   {
      c_TXBUF2[ui_TX2wr] = *pc_str;
      ui_TX2wr++;
      if (ui_TX2wr > BUF2_LAENGE) ui_TX2wr = 0;
      SREG &= 0x7F;  //INTs sperren  
      ui_TX2cnt++; 
      if (ui_TX2cnt > BUF2_LAENGE) ui_TX2cnt = 0;
      SREG |= 0x80;    //INTs freigeben  
  	  pc_str++; 
      watchdog_reset();
   }
   
  UCSR2B |= 0x20; 	 
}


//*****************************************************************************
//	UART - Zeichen empfangen
//	Input:	- 
//	Output:	char  Empfangszeichen
//*****************************************************************************
char UART2_RX(void)
{      
char uc_x;
         
   while(ui_RX2cnt == 0)		//warte auf Zeichen
   {
      watchdog_reset();
   }

   uc_x = c_RXBUF2[ui_RX2rd];
  
   SREG &= 0x7F;  //INTs sperren  
   ui_RX2rd++;
   if (ui_RX2rd > BUF2_LAENGE) ui_RX2rd = 0;
   if (ui_RX2cnt > 0) ui_RX2cnt--;  
   SREG |= 0x80;  //INTs freigeben  
  
   return(uc_x);
}


