//*****************************************************************************
//
//	UART3 des ATmega Interrupt gesteuert
//
//*****************************************************************************
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdlib.h>
#include <avr/wdt.h>
#include <string.h>
#include "UART3.h"
#include "watchdog.h"


#define BUF3_LAENGE  128

char c_RXBUF3[BUF3_LAENGE+1];
char c_TXBUF3[BUF3_LAENGE+1];
  
unsigned int ui_RX3cnt,ui_TX3cnt,ui_RX3wr,ui_TX3wr,ui_RX3rd,ui_TX3rd;


//*****************************************************************************
//	RX - INT
//*****************************************************************************
ISR( USART3_RX_vect)
{
   c_RXBUF3[ui_RX3wr]  = UDR3;
   ui_RX3wr++;
  
   if (ui_RX3wr > BUF3_LAENGE) ui_RX3wr = 0;
   ui_RX3cnt++; 
}

//*****************************************************************************
//	TX - INT  USART_UDRE_vect
//*****************************************************************************
ISR (USART3_UDRE_vect )
{
  if (ui_TX3cnt)
   {
      UDR3 = c_TXBUF3[ui_TX3rd];
	  ui_TX3rd++;
	  if (ui_TX3rd > BUF3_LAENGE) ui_TX3rd = 0;
	  if (ui_TX3cnt > 0) ui_TX3cnt--; 
   }
//   else UCSR3B &= 0xDF;
}


//*****************************************************************************
//	UART - Initialisierung
//	Input:	-
//	Output:	-
//*****************************************************************************
void UART3_INIT(void)
{
unsigned char x;
unsigned int i;

   SREG &= 0x7F;				//global INT aus
	                   
   ui_RX3cnt = 0;
   ui_TX3cnt = 0;
   ui_RX3wr  = 0;
   ui_TX3wr  = 0;
   ui_RX3rd  = 0;
   ui_TX3rd  = 0;
	
   for(i=0;i<BUF3_LAENGE+1;i++)
	{
	   c_TXBUF3[i] = 0;
	   c_RXBUF3[i] = 0;
       watchdog_reset();
	}
	
	// DDRE 	= 0xFE;  
	// PORTE 	= 0x00; 	

   UCSR3A = 0;    //Int Flag loeschen
   UCSR3B = 0;
   UCSR3C = 0x0E;
   UBRR3  = 103;				// 9k6 Baudrate (103 bei 16MHz)
   //UBRR3  =  25;				//38k4 Baudrate ( 25 bei 16MHz)

   UCSR3B = 0x98;				//RX & TX der UART an, Interrupts enabled	
	
   x = UDR3;

   while((UCSR3A & 0x20) == 0) //UDRE == 0 ?
    {
       watchdog_reset();
    }

   while((UCSR3A & 0x40) == 0x40) //TXC  == 1 ?
    {
       watchdog_reset();
    }

   UDR3 = 0x0D;					//sende CR, damit TXC flag setzen
   SREG |= 0x80;				//global INT an
}
 
//*****************************************************************************
//	UART - Test, ob Zeichen empfangen
//	Input:	-
//	Output:	1 ja , 0 nein
//*****************************************************************************
unsigned char UART3_RXTest(void)
{
   if (ui_RX3cnt) return(1); 
   	 else return(0);
}


//*****************************************************************************
//	UART - Zeichen senden
//	Input:	char  Zeichen
//	Output:	-
//*****************************************************************************
void UART3_TX(char c_x)
{
   while(ui_TX3cnt > 0) //warte bis Puffer leer
   {
      watchdog_reset();
   }

   SREG &= 0x7F;  //INTs sperren  

   c_TXBUF3[ui_TX3wr] = c_x;
   ui_TX3wr++;
   if (ui_TX3wr > BUF3_LAENGE) ui_TX3wr = 0;
   ui_TX3cnt++; 
   if (ui_TX3cnt > BUF3_LAENGE) ui_TX3cnt = 0; 

   SREG |= 0x80;  //INTs freigeben  

   UCSR3B |= 0x20; 	     
}


//*****************************************************************************
//	UART - String senden
//	Input:	char* String 
//	Output:	-
//*****************************************************************************
void UART3_TXStr(char * pc_str)
{
   while(ui_TX3cnt > 0) //warte bis Puffer leer
   {
      watchdog_reset();
   }

   while (*pc_str != '\0')
   {
      c_TXBUF3[ui_TX3wr] = *pc_str;
      ui_TX3wr++;
      if (ui_TX3wr > BUF3_LAENGE) ui_TX3wr = 0;
      SREG &= 0x7F;  //INTs sperren  
      ui_TX3cnt++; 
      if (ui_TX3cnt > BUF3_LAENGE) ui_TX3cnt = 0;
      SREG |= 0x80;    //INTs freigeben  
  	  pc_str++; 
      watchdog_reset();
   }
   
  UCSR3B |= 0x20; 	 
}


//*****************************************************************************
//	UART - Zeichen empfangen
//	Input:	- 
//	Output:	char  Empfangszeichen
//*****************************************************************************
char UART3_RX(void)
{      
char uc_x;
         
   while(ui_RX3cnt == 0)		//warte auf Zeichen
   {
      watchdog_reset();
   }

   uc_x = c_RXBUF3[ui_RX3rd];
  
   SREG &= 0x7F;  //INTs sperren  
   ui_RX3rd++;
   if (ui_RX3rd > BUF3_LAENGE) ui_RX3rd = 0;
   if (ui_RX3cnt > 0) ui_RX3cnt--;  
   SREG |= 0x80;  //INTs freigeben  
  
   return(uc_x);
}


